@@ -10,16 +10,21 @@
  */
 
 
-#include <qdir.h>
-#include <qcolor.h>
-#include <qlabel.h>
-#include <qlayout.h>
-#include <qfile.h>
-#include <qpaintdevicemetrics.h>
-#include <qcheckbox.h>
-#include <qcombobox.h>
-#include <qspinbox.h>
-#include <qframe.h>
+#include <QDir>
+#include <QColor>
+#include <QLabel>
+#include <QLayout>
+#include <QFile>
+#include <QFileInfo>
+#include <QCheckBox>
+#include <QComboBox>
+#include <QSpinBox>
+#include <QDesktopWidget>
+#include <QImage>
+#include <QPainter>
+#include <QPixmap>
+#include <QPolygon>
+
 
 #include <kconfig.h>
 #include <kglobal.h>
@@ -31,8 +36,9 @@
 #include <kstandarddirs.h>
 #include <kurlrequester.h>
 #include <kaboutdata.h>
-#include <kaboutapplication.h>
+#include <kaboutapplicationdialog.h>
 #include <kdebug.h>
+#include <krandom.h>
 
 #include <stdlib.h>
 #include <assert.h>
@@ -41,30 +47,44 @@
 
 #include "slideshow.h"
 #include "slideshow.moc"
-#include "slideshowcfg.h"
 
 
 #define SLIDESHOW_VERSION "2.3.0"
 static const char version[] = SLIDESHOW_VERSION;
 static const char description[] = I18N_NOOP("KSlideshow");
 
+static KAboutData* s_aboutData = 0;
 
 // libkscreensaver interface
-extern "C"
+class KSlideShowSaverInterface : public KScreenSaverInterface
 {
-    const char *kss_applicationName = "kslideshow.kss";
-    const char *kss_description = I18N_NOOP( "KSlideshow" );
-    const char *kss_version = SLIDESHOW_VERSION;
+public:
+    virtual KAboutData* aboutData() {
+        return s_aboutData;
+    }
 
-    KScreenSaver *kss_create( WId id )
+    virtual KScreenSaver* create( WId id )
     {
         return new kSlideShowSaver( id );
     }
 
-    QDialog *kss_setup()
+    virtual QDialog* setup()
     {
         return new kSlideShowSetup();
     }
+};
+
+int main( int argc, char *argv[] )
+{
+    s_aboutData = new KAboutData( "kslideshow.kss", 0, ki18n("SlideShow"),
+                                  version, ki18n(description), KAboutData::License_GPL,
+                                  ki18n("(c) 1999-2003, The KDE Team") );
+    s_aboutData->addAuthor(ki18n("Stefan Taferner"), KLocalizedString(), "taferner@kde.org");
+    s_aboutData->addAuthor(ki18n("Chris Howells"), KLocalizedString(), "howells@kde.org");
+    s_aboutData->addAuthor(ki18n("Sven Leiber"), KLocalizedString(), "s.leiber@web.de");
+
+    KSlideShowSaverInterface kss;
+    return kScreenSaverMain( argc, argv, kss );
 }
 
 
@@ -73,15 +93,6 @@ extern "C"
 //=============================================================================
 kSlideShowSaver::kSlideShowSaver( WId id ): KScreenSaver(id)
 {
-  time_t t;
-
-  time(&t);
-  srand((unsigned int)t);
-
-  KImageIO::registerFormats();
-
-  blank();
-
   mEffect = NULL;
   mNumEffects = 0;
   mIntArray = NULL;
@@ -91,14 +102,28 @@ kSlideShowSaver::kSlideShowSaver( WId id ): KScreenSaver(id)
   initNextScreen();
 
   mFileIdx = 0;
-  mColorContext = QColor::enterAllocContext();
 
   mEffectRunning = false;
-  loadNextImage();
-  createNextScreen();
 
-  mTimer.start(10, true);
+  mTimer.setSingleShot(true);
   connect(&mTimer, SIGNAL(timeout()), SLOT(slotTimeout()));
+  mTimer.start(10);
+
+  QDesktopWidget *d = QApplication::desktop();
+  if(geometry() == d->geometry() && d->numScreens() > 1)
+  {
+    for(int i = 0; i < d->numScreens(); ++i)
+    {
+      QRect s = d->screenGeometry(i);
+      mGeoList.append(mScreenGeo(s.width(), s.height(), s.topLeft().x(), s.topLeft().y()));
+    }
+  }
+  else
+  {
+    mGeoList.append(mScreenGeo(width(), height(), 0, 0));
+  }
+  setAttribute(Qt::WA_NoSystemBackground);
+  createNextScreen();
 }
 
 
@@ -107,38 +132,31 @@ kSlideShowSaver::~kSlideShowSaver()
 {
   delete [] mIntArray;
   delete [] mEffectList;
-
-  mTimer.stop();
-  if (mPainter.isActive()) mPainter.end();
-  QColor::leaveAllocContext();
-  QColor::destroyAllocContext(mColorContext);
 }
 
 
 //-----------------------------------------------------------------------------
 void kSlideShowSaver::initNextScreen()
 {
-  QPaintDeviceMetrics metric(this);
   int w, h;
 
   w = width();
   h = height();
-  mNextScreen = QPixmap(w, h, metric.depth());
+  mNextScreen = QPixmap(w, h);
 }
 
 
 //-----------------------------------------------------------------------------
 void kSlideShowSaver::readConfig()
 {
-  KConfig *config = KGlobal::config();
-  config->setGroup("Settings");
-  mShowRandom = config->readBoolEntry("ShowRandom", true);
-  mZoomImages = config->readBoolEntry("ZoomImages", false);
-  mPrintName = config->readBoolEntry("PrintName", true);
-  mDirectory = config->readPathEntry("Directory", KGlobal::dirs()->findDirs("wallpaper", "").last());
-  mDelay = config->readNumEntry("Delay", 10) * 1000;
-  mSubdirectory = config->readBoolEntry("SubDirectory", false);
-  mRandomPosition = config->readBoolEntry("RandomPosition", false);
+  KConfigGroup config(KGlobal::config(), "Settings");
+  mShowRandom = config.readEntry("ShowRandom", true);
+  mZoomImages = config.readEntry("ZoomImages", false);
+  mPrintName = config.readEntry("PrintName", true);
+  mDirectory = config.readPathEntry("Directory", KGlobal::dirs()->findDirs("wallpaper", "").last());
+  mDelay = config.readEntry("Delay", 10) * 1000;
+  mSubdirectory = config.readEntry("SubDirectory", false);
+  mRandomPosition = config.readEntry("RandomPosition", false);
 
   loadDirectory();
 }
@@ -173,11 +191,10 @@ int kSlideShowSaver::effectMultiCircleOut(bool aInit)
 {
   int x, y, i;
   double alpha;
-  static QPointArray pa(4);
+  static QPolygon pa(4);
 
   if (aInit)
   {
-    startPainter();
     mw = width();
     mh = height();
     mx = mw;
@@ -185,7 +202,7 @@ int kSlideShowSaver::effectMultiCircleOut(bool aInit)
     pa.setPoint(0, mw>>1, mh>>1);
     pa.setPoint(3, mw>>1, mh>>1);
     mfy = sqrt((double)mw*mw + mh*mh) / 2;
-    mi  = rand()%15 + 2;
+    mi  = KRandom::random()%15 + 2;
     mfd = M_PI*2/mi;
     mAlpha = mfd;
     mwait = 10 * mi;
@@ -194,11 +211,16 @@ int kSlideShowSaver::effectMultiCircleOut(bool aInit)
 
   if (mAlpha < 0)
   {
-    mPainter.end();
     showNextScreen();
     return -1;
   }
 
+  QPainter p(this);
+  QBrush brush;
+  brush.setTexture(mNextScreen);
+  p.setBrush(brush);
+  p.setPen(Qt::NoPen);
+
   for (alpha=mAlpha, i=mi; i>=0; i--, alpha+=mfd)
   {
     x = (mw>>1) + (int)(mfy * cos(-alpha));
@@ -210,7 +232,7 @@ int kSlideShowSaver::effectMultiCircleOut(bool aInit)
     pa.setPoint(1, x, y);
     pa.setPoint(2, mx, my);
 
-    mPainter.drawPolygon(pa);
+    p.drawPolygon(pa);
   }
   mAlpha -= mfx;
 
@@ -223,7 +245,6 @@ int kSlideShowSaver::effectSpiralIn(bool aInit)
 {
   if (aInit)
   {
-    startPainter();
     mw = width();
     mh = height();
     mix = mw / 8;
@@ -242,7 +263,6 @@ int kSlideShowSaver::effectSpiralIn(bool aInit)
 
   if (mi==0 && mx0>=mx1)
   {
-    mPainter.end();
     showNextScreen();
     return -1;
   }
@@ -276,7 +296,8 @@ int kSlideShowSaver::effectSpiralIn(bool aInit)
     my0 += miy;
   }
 
-  bitBlt(this, mx, my, &mNextScreen, mx, my, mix, miy, CopyROP, true);
+  QPainter p(this);
+  p.drawPixmap(mx, my, mNextScreen, mx, my, mix, miy);
 
   mx += mdx;
   my += mdy;
@@ -306,14 +327,15 @@ int kSlideShowSaver::effectMeltdown(bool aInit)
   }
 
   done = true;
+
+  QPainter p(this);
   for (i=0,x=0; i<mix; i++,x+=mdx)
   {
     y = mIntArray[i];
     if (y >= mh) continue;
     done = false;
-    if ((rand()&15) < 6) continue;
-    bitBlt(this, x, y+mdy, this, x, y, mdx, mh-y-mdy, CopyROP, true);
-    bitBlt(this, x, y, &mNextScreen, x, y, mdx, mdy, CopyROP, true);
+    if ((KRandom::random()&15) < 6) continue;
+    p.drawPixmap(x, y, mNextScreen, x, y, mdx, mdy);
     mIntArray[i] += mdy;
   }
 
@@ -332,11 +354,10 @@ int kSlideShowSaver::effectMeltdown(bool aInit)
 int kSlideShowSaver::effectCircleOut(bool aInit)
 {
   int x, y;
-  static QPointArray pa(4);
+  static QPolygon pa(4);
 
   if (aInit)
   {
-    startPainter();
     mw = width();
     mh = height();
     mx = mw;
@@ -350,7 +371,6 @@ int kSlideShowSaver::effectCircleOut(bool aInit)
 
   if (mAlpha < 0)
   {
-    mPainter.end();
     showNextScreen();
     return -1;
   }
@@ -364,7 +384,13 @@ int kSlideShowSaver::effectCircleOut(bool aInit)
   pa.setPoint(1, x, y);
   pa.setPoint(2, mx, my);
 
-  mPainter.drawPolygon(pa);
+  QBrush brush;
+  brush.setTexture(mNextScreen);
+  QPainter p(this);
+  p.setBrush(brush);
+  p.setPen(Qt::NoPen);
+
+  p.drawPolygon(pa);
 
   return 20;
 }
@@ -379,7 +405,7 @@ int kSlideShowSaver::effectSweep(bool aInit)
   {
     // subtype: 0=sweep right to left, 1=sweep left to right
     //          2=sweep bottom to top, 3=sweep top to bottom
-    mSubType = rand() % 4;
+    mSubType = KRandom::random() % 4;
     mw  = width();
     mh  = height();
     mdx = (mSubType==1 ? 16 : -16);
@@ -388,6 +414,7 @@ int kSlideShowSaver::effectSweep(bool aInit)
     my  = (mSubType==3 ? 0 : mh);
   }
 
+  QPainter p(this);
   if (mSubType==0 || mSubType==1)
   {
     // horizontal sweep
@@ -398,7 +425,7 @@ int kSlideShowSaver::effectSweep(bool aInit)
     }
     for (w=2,i=4,x=mx; i>0; i--, w<<=1, x-=mdx)
     {
-      bitBlt(this, x, 0, &mNextScreen, x, 0, w, mh, CopyROP, true);
+      p.drawPixmap(x, 0, mNextScreen, x, 0, w, mh);
     }
     mx += mdx;
   }
@@ -412,7 +439,7 @@ int kSlideShowSaver::effectSweep(bool aInit)
     }
     for (h=2,i=4,y=my; i>0; i--, h<<=1, y-=mdy)
     {
-      bitBlt(this, 0, y, &mNextScreen, 0, y, mw, h, CopyROP, true);
+      p.drawPixmap(0, y, mNextScreen, 0, y, mw, h);
     }
     my += mdy;
   }
@@ -428,7 +455,6 @@ int kSlideShowSaver::effectBlobs(bool aInit)
 
   if (aInit)
   {
-    startPainter();
     mAlpha = M_PI * 2;
     mw = width();
     mh = height();
@@ -437,16 +463,21 @@ int kSlideShowSaver::effectBlobs(bool aInit)
 
   if (mi <= 0)
   {
-    mPainter.end();
     showNextScreen();
     return -1;
   }
 
-  mx = rand() % mw;
-  my = rand() % mh;
-  r = (rand() % 200) + 50;
+  mx = KRandom::random() % mw;
+  my = KRandom::random() % mh;
+  r = (KRandom::random() % 200) + 50;
+
+  QBrush brush;
+  brush.setTexture(mNextScreen);
+  QPainter p(this);
+  p.setBrush(brush);
+  p.setPen(Qt::NoPen);
+  p.drawEllipse(mx-r, my-r, r, r);
 
-  mPainter.drawEllipse(mx-r, my-r, r, r);
   mi--;
 
   return 10;
@@ -458,18 +489,20 @@ int kSlideShowSaver::effectRandom(bool /*aInit*/)
 {
   int x, y, i, w, h, fact, sz;
 
-  fact = (rand() % 3) + 1;
+  fact = (KRandom::random() % 3) + 1;
 
   w = width() >> fact;
   h = height() >> fact;
   sz = 1 << fact;
 
+  QPainter p(this);
   for (i = (w*h)<<1; i > 0; i--)
   {
-    x = (rand() % w) << fact;
-    y = (rand() % h) << fact;
-    bitBlt(this, x, y, &mNextScreen, x, y, sz, sz, CopyROP, true);
+    x = (KRandom::random() % w) << fact;
+    y = (KRandom::random() % h) << fact;
+    p.drawPixmap(x, y, mNextScreen, x, y, sz, sz);
   }
+  p.end();
   showNextScreen();
 
   return -1;
@@ -500,8 +533,11 @@ int kSlideShowSaver::effectGrowing(bool aInit)
     return -1;
   }
 
-  bitBlt(this, mx, my, &mNextScreen, mx, my,
-	 mw - (mx<<1), mh - (my<<1), CopyROP, true);
+  if((mw - (mx<<1) == 0) && (mh - (my<<1) == 0))
+    return 1;
+
+  QPainter p(this);
+  p.drawPixmap(mx, my, mNextScreen, mx, my, mw - (mx<<1), mh - (my<<1));
 
   return 20;
 }
@@ -537,12 +573,11 @@ int kSlideShowSaver::effectChessboard(bool aInit)
   miy = miy ? 0 : mdy;
   my  = my ? 0 : mdy;
 
+  QPainter p(this);
   for (y=0; y<mw; y+=(mdy<<1))
   {
-    bitBlt(this, mix, y+miy, &mNextScreen, mix, y+miy,
-	   mdx, mdy, CopyROP, true);
-    bitBlt(this, mx, y+my, &mNextScreen, mx, y+my,
-	   mdx, mdy, CopyROP, true);
+    p.drawPixmap(mix, y+miy, mNextScreen, mix, y+miy, mdx, mdy);
+    p.drawPixmap(mx, y+my, mNextScreen, mx, y+my, mdx, mdy);
   }
 
   return mwait;
@@ -563,7 +598,7 @@ int kSlideShowSaver::effectIncomingEdges(bool aInit)
     mfx = mix / 100.0;
     mfy = miy / 100.0;
     mi = 0;
-    mSubType = rand() & 1;
+    mSubType = KRandom::random() & 1;
   }
 
   mx = (int)(mfx * mi);
@@ -579,21 +614,25 @@ int kSlideShowSaver::effectIncomingEdges(bool aInit)
   y1 = mh - my;
   mi++;
 
+  if((mx==0) && (my==0))
+    return 1; // otherwise drawPixmap draws the bottom-right of mNextScreen
+
+  QPainter p(this);
   if (mSubType)
   {
     // moving image edges
-    bitBlt(this,  0,  0, &mNextScreen, mix-mx, miy-my, mx, my, CopyROP, true);
-    bitBlt(this, x1,  0, &mNextScreen, mix, miy-my, mx, my, CopyROP, true);
-    bitBlt(this,  0, y1, &mNextScreen, mix-mx, miy, mx, my, CopyROP, true);
-    bitBlt(this, x1, y1, &mNextScreen, mix, miy, mx, my, CopyROP, true);
+    p.drawPixmap(0, 0, mNextScreen, mix-mx, miy-my, mx, my);
+    p.drawPixmap(x1, 0, mNextScreen, mix, miy-my, mx, my);
+    p.drawPixmap(0, y1, mNextScreen, mix-mx, miy, mx, my);
+    p.drawPixmap(x1, y1, mNextScreen, mix, miy, mx, my);
   }
   else
   {
     // fixed image edges
-    bitBlt(this,  0,  0, &mNextScreen,  0,  0, mx, my, CopyROP, true);
-    bitBlt(this, x1,  0, &mNextScreen, x1,  0, mx, my, CopyROP, true);
-    bitBlt(this,  0, y1, &mNextScreen,  0, y1, mx, my, CopyROP, true);
-    bitBlt(this, x1, y1, &mNextScreen, x1, y1, mx, my, CopyROP, true);
+    p.drawPixmap(0, 0, mNextScreen,  0,  0, mx, my);
+    p.drawPixmap(x1, 0, mNextScreen, x1,  0, mx, my);
+    p.drawPixmap(0, y1, mNextScreen,  0, y1, mx, my);
+    p.drawPixmap(x1, y1, mNextScreen, x1, y1, mx, my);
   }
   return 20;
 }
@@ -614,9 +653,10 @@ int kSlideShowSaver::effectHorizLines(bool aInit)
 
   if (iyPos[mi] < 0) return -1;
 
+  QPainter p(this);
   for (y=iyPos[mi]; y<mh; y+=8)
   {
-    bitBlt(this, 0, y, &mNextScreen, 0, y, mw, 1, CopyROP, true);
+    p.drawPixmap(0, y, mNextScreen, 0, y, mw, 1);
   }
 
   mi++;
@@ -640,9 +680,10 @@ int kSlideShowSaver::effectVertLines(bool aInit)
 
   if (ixPos[mi] < 0) return -1;
 
+  QPainter p(this);
   for (x=ixPos[mi]; x<mw; x+=8)
   {
-    bitBlt(this, x, 0, &mNextScreen, x, 0, 1, mh, CopyROP, true);
+    p.drawPixmap(x, 0, mNextScreen, x, 0, 1, mh);
   }
 
   mi++;
@@ -651,30 +692,24 @@ int kSlideShowSaver::effectVertLines(bool aInit)
 }
 
 
-//-----------------------------------------------------------------------------
-void kSlideShowSaver::startPainter(Qt::PenStyle aPen)
-{
-  QBrush brush;
-  brush.setPixmap(mNextScreen);
-  if (mPainter.isActive()) mPainter.end();
-  mPainter.begin(this);
-  mPainter.setBrush(brush);
-  mPainter.setPen(aPen);
-}
-
-
 //-----------------------------------------------------------------------------
 void kSlideShowSaver::restart()
 {
   mEffectRunning = false;
   mEffect = NULL;
-  blank();
-  slotTimeout();
+  update();
 }
 
 
 //-----------------------------------------------------------------------------
 void kSlideShowSaver::slotTimeout()
+{
+  update();
+}
+
+
+//-----------------------------------------------------------------------------
+void kSlideShowSaver::paintEvent(QPaintEvent *)
 {
   int tmout = -1;
   int i;
@@ -688,7 +723,7 @@ void kSlideShowSaver::slotTimeout()
     loadNextImage();
     createNextScreen();
 
-    if (mNumEffects > 1) i = rand() % mNumEffects;
+    if (mNumEffects > 1) i = KRandom::random() % mNumEffects;
     else i = 0;
 
     mEffect = mEffectList[i];
@@ -700,15 +735,15 @@ void kSlideShowSaver::slotTimeout()
     tmout = mDelay;
     mEffectRunning = false;
   }
-  mTimer.start(tmout, true);
+  mTimer.start(tmout);
 }
 
 
 //----------------------------------------------------------------------------
 void kSlideShowSaver::showNextScreen()
 {
-  bitBlt(this, 0, 0, &mNextScreen, 0, 0,
-	 mNextScreen.width(), mNextScreen.height(), CopyROP, true);
+  QPainter p(this);
+  p.drawPixmap(0, 0, mNextScreen, 0, 0, mNextScreen.width(), mNextScreen.height());
 }
 
 
@@ -720,86 +755,92 @@ void kSlideShowSaver::createNextScreen()
   double fx, fy;
 
   if (mNextScreen.size() != size())
-    mNextScreen.resize(size());
+    mNextScreen = QPixmap(size());
 
-  mNextScreen.fill(black);
-
-  ww = mNextScreen.width();
-  wh = mNextScreen.height();
-  iw = mImage.width();
-  ih = mImage.height();
+  mNextScreen.fill(Qt::black);
 
   p.begin(&mNextScreen);
+  
+  foreach( const mScreenGeo& geo, mGeoList )
+  {
+    loadNextImage();
+    
+    iw = mImage.width();
+    ih = mImage.height();
+    ww = geo.mW;
+    wh = geo.mH;
 
-  if (mFileList.isEmpty())
-  {
-    p.setPen(QColor("white"));
-    p.drawText(20 + (rand() % (ww>>1)), 20 + (rand() % (wh>>1)),
-	       i18n("No images found"));
-  }
-  else
-  {
-    if (mZoomImages)
+    if (mFileList.isEmpty())
     {
-      fx = (double)ww / iw;
-      fy = (double)wh / ih;
-      if (fx > fy) fx = fy;
-      if (fx > 2) fx = 2;
-      iw = (int)(iw * fx);
-      ih = (int)(ih * fx);
-      QImage scaledImg = mImage.smoothScale(iw, ih);
-
-      x = (ww - iw) >> 1;
-      y = (wh - ih) >> 1;
-
-      p.drawImage(x, y, scaledImg);
+      p.setPen(QColor("white"));
+      p.drawText(20 + (KRandom::random() % (ww>>1)), 20 + (KRandom::random() % (wh>>1)),
+	         i18n("No images found"));
     }
     else
     {
-      if(iw > ww || ih > wh)
+      if (mZoomImages)
       {
         fx = (double)ww / iw;
-	fy = (double)wh / ih;
-	if (fx > fy) fx = fy;
-	if (fx > 2) fx = 2;
-	iw = (int)(iw * fx);
-	ih = (int)(ih * fx);
-	QImage scaledImg = mImage.smoothScale(iw, ih);
+        fy = (double)wh / ih;
+        if (fx > fy) fx = fy;
+        if (fx > 2) fx = 2;
+        iw = (int)(iw * fx);
+        ih = (int)(ih * fx);
+        QImage scaledImg = mImage.scaled(iw, ih,
+            Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
 
-	x = (ww - iw) >> 1;
-	y = (wh - ih) >> 1;
+        x = ((ww - iw) >> 1) + geo.mXorg;
+        y = ((wh - ih) >> 1) + geo.mYorg;
 
-	p.drawImage(x, y, scaledImg);
+        p.drawImage(x, y, scaledImg);
       }
       else
       {
-        if(mRandomPosition)
+        if(iw >= ww || ih >= wh)
         {
-          x = rand() % (ww - iw);
-          y = rand() % (wh - ih);
+          fx = (double)ww / iw;
+	  fy = (double)wh / ih;
+	  if (fx > fy) fx = fy;
+	  if (fx > 2) fx = 2;
+	  iw = (int)(iw * fx);
+	  ih = (int)(ih * fx);
+          QImage scaledImg = mImage.scaled(iw, ih,
+              Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+
+          x = ((ww - iw) >> 1) + geo.mXorg;
+          y = ((wh - ih) >> 1) + geo.mYorg;
+
+	  p.drawImage(x, y, scaledImg);
         }
         else
         {
-          x = (ww - iw) >> 1;
-          y = (wh - ih) >> 1;
-        }
+          if(mRandomPosition)
+          {
+            x = (KRandom::random() % (ww - iw)) + geo.mXorg;
+            y = (KRandom::random() % (wh - ih)) + geo.mYorg;
+          }
+          else
+          {
+            x = ((ww - iw) >> 1) + geo.mXorg;
+            y = ((wh - ih) >> 1) + geo.mYorg;
+          }
 
-        // bitBlt(&mNextScreen, x, y, &mImage, 0, 0, iw, ih, CopyROP, false);
-        p.drawImage(x, y, mImage);
+          // bitBlt(&mNextScreen, x, y, &mImage, 0, 0, iw, ih);
+          p.drawImage(x, y, mImage);
+        }
+      }
+
+      if (mPrintName)
+      {
+        p.setPen(QColor("black"));
+        for (x=9; x<=11; x++)
+	  for (y=21; y>=19; y--)
+            p.drawText(x + geo.mXorg, wh-y+geo.mYorg, mImageName);
+        p.setPen(QColor("white"));
+        p.drawText(10 + geo.mXorg, wh-20 + geo.mYorg, mImageName);
       }
     }
-
-    if (mPrintName)
-    {
-      p.setPen(QColor("black"));
-      for (x=9; x<=11; x++)
-	for (y=21; y>=19; y--)
-	  p.drawText(x, wh-y, mImageName);
-      p.setPen(QColor("white"));
-      p.drawText(10, wh-20, mImageName);
-    }
   }
-
   p.end();
 }
 
@@ -810,6 +851,8 @@ void kSlideShowSaver::loadNextImage()
   QString fname;
   int num;
 
+nexttry:
+
   num = mFileList.count();
   if (num <= 0) //no files in the directory
   {
@@ -818,9 +861,8 @@ void kSlideShowSaver::loadNextImage()
 
   if (mShowRandom)
   {
-    mFileIdx = rand() % num;
-    fname = mFileList[mFileIdx];
-    mFileList.remove(fname);
+    mFileIdx = KRandom::random() % num;
+    fname = mFileList.takeAt(mFileIdx);
     if (num == 1) //we're about to run out of images
     {
       mFileList = mRandomList;
@@ -834,21 +876,14 @@ void kSlideShowSaver::loadNextImage()
 
   if (!mImage.load(fname))
   {
-    kdDebug() << "Failed to load image " << fname << endl;
-    mFileList.remove(fname);
-    mRandomList.remove(fname);
-    if (!mFileList.isEmpty())
-        loadNextImage();
-    return;
+    kDebug() << "Failed to load image " << fname;
+    mFileList.removeAll(fname);
+    mRandomList.removeAll(fname);
+    goto nexttry;
   }
   mFileIdx++;
 
-  int i, j;
-  i = fname.findRev('.');
-  if (i < 0) i = 32767;
-  j = fname.findRev('/') + 1;
-  if (j < 0) j = 0;
-  mImageName = fname.mid(j, i-j);
+  mImageName = QFileInfo(fname).baseName();
 }
 
 
@@ -865,69 +900,64 @@ void kSlideShowSaver::traverseDirectory(const QString &dirName)
 {
   QDir dir(dirName);
   if (!dir.exists())
-    return;
+  {
+     return ;
+  }
   dir.setFilter(QDir::Dirs | QDir::Files);
 
-  const QFileInfoList *fileinfolist = dir.entryInfoList();
-  QFileInfoListIterator it(*fileinfolist);
-  QFileInfo *fi;
-  while ((fi = it.current()))
+  QFileInfoList fileinfolist = dir.entryInfoList();
+  QFileInfoListIterator it = fileinfolist.begin();
+  while ( it != fileinfolist.end())
   {
-     if (fi->fileName() == "." || fi->fileName() == "..")
+     if (it->fileName() == "." || it->fileName() == "..")
      {
         ++it;
         continue;
      }
-     if (fi->isDir() && fi->isReadable() && mSubdirectory)
+     if (it->isDir() && it->isReadable() && mSubdirectory)
      {
-        traverseDirectory(fi->filePath());
+        traverseDirectory(it->filePath());
      }
      else
      {
-        if (!fi->isDir())
+        if (!it->isDir())
         {
-           mFileList.append(fi->filePath().data());
+           mFileList.append(it->filePath());
         }
      }
-  ++it;
-   }
-}
-
-
-//-----------------------------------------------------------------------------
-void kSlideShowSaver::blank()
-{
-   erase();
+     ++it;
+  }
 }
 
 
 //=============================================================================
 //  Class kSlideShowSetup
 //=============================================================================
-kSlideShowSetup::kSlideShowSetup(QWidget *aParent, const char *aName)
-  : KDialogBase(aParent, aName, true, i18n( "Setup Slide Show Screen Saver" ),
-    Ok|Cancel|Help, Ok, true )
+kSlideShowSetup::kSlideShowSetup(QWidget *aParent)
+  : KDialog(aParent)
 {
+  setCaption(i18n( "Setup Slide Show Screen Saver" ));
+  setButtons(Ok|Cancel|Help);
+  setDefaultButton(Ok);
+  setModal(true);
   setButtonText( Help, i18n( "A&bout" ) );
 
-  QWidget *main = makeMainWidget();
-  QVBoxLayout *top = new QVBoxLayout( main, 0, spacingHint() );
-
-  cfg = new SlideShowCfg( main, "SlideShowCfg" );
-  top->addWidget( cfg );
-  top->addStretch();
+  QWidget *main = new QWidget(this);
+  setMainWidget(main);
+  cfg = new SlideShowCfg();
+  cfg->setupUi( main );
 
   cfg->mPreview->setFixedSize(220, 170);
-  cfg->mPreview->setBackgroundColor(black);
   cfg->mPreview->show();    // otherwise saver does not get correct size
   mSaver = new kSlideShowSaver(cfg->mPreview->winId());
 
   cfg->mDirChooser->setMode(KFile::Directory | KFile::ExistingOnly);
   connect(cfg->mDirChooser, SIGNAL(returnPressed(const QString &)),
       SLOT(slotDirSelected(const QString &)));
-  connect(cfg->mDirChooser, SIGNAL(urlSelected(const QString &)),
-      SLOT(slotDirSelected(const QString &)));
-
+  connect(cfg->mDirChooser, SIGNAL(urlSelected(const KUrl &)),
+      SLOT(slotDirSelected(const KUrl &)));
+  connect(this,SIGNAL(okClicked()),this,SLOT(slotOk()));
+  connect(this,SIGNAL(helpClicked()),this,SLOT(slotHelp()));
   readSettings();
 }
 
@@ -939,34 +969,32 @@ kSlideShowSetup::~kSlideShowSetup()
 //-----------------------------------------------------------------------------
 void kSlideShowSetup::readSettings()
 {
-  KConfig *config = KGlobal::config();
+    KConfigGroup config( KGlobal::config(), "Settings");
 
-  config->setGroup("Settings");
-  cfg->mCbxRandom->setChecked(config->readBoolEntry("ShowRandom", true));
-  cfg->mCbxZoom->setChecked(config->readBoolEntry("ZoomImages", false));
-  cfg->mCbxShowName->setChecked(config->readBoolEntry("PrintName", true));
-  cfg->mDelay->setValue(config->readNumEntry("Delay", 20));
-  cfg->mDirChooser->setURL(config->readPathEntry("Directory"));
-  cfg->mCbxSubdirectory->setChecked(config->readBoolEntry("SubDirectory", false));
-  cfg->mCbxRandomPosition->setChecked(config->readBoolEntry("RandomPosition", false));
+    cfg->mCbxRandom->setChecked(config.readEntry("ShowRandom", true));
+    cfg->mCbxZoom->setChecked(config.readEntry("ZoomImages", false));
+    cfg->mCbxShowName->setChecked(config.readEntry("PrintName", true));
+    cfg->mDelay->setValue(config.readEntry("Delay", 20));
+    cfg->mDirChooser->setPath(config.readPathEntry("Directory", QString()));
+    cfg->mCbxSubdirectory->setChecked(config.readEntry("SubDirectory", false));
+    cfg->mCbxRandomPosition->setChecked(config.readEntry("RandomPosition", false));
 }
 
 
 //-----------------------------------------------------------------------------
 void kSlideShowSetup::writeSettings()
 {
-  KConfig *config = KGlobal::config();
+  KConfigGroup config( KGlobal::config(), "Settings");
 
-  config->setGroup("Settings");
-  config->writeEntry("ShowRandom", cfg->mCbxRandom->isChecked());
-  config->writeEntry("ZoomImages", cfg->mCbxZoom->isChecked());
-  config->writeEntry("PrintName",  cfg->mCbxShowName->isChecked());
-  config->writeEntry("Delay", cfg->mDelay->value());
-  config->writePathEntry("Directory", cfg->mDirChooser->url());
-  config->writeEntry("SubDirectory", cfg->mCbxSubdirectory->isChecked());
-  config->writeEntry("RandomPosition", cfg->mCbxRandomPosition->isChecked());
+  config.writeEntry("ShowRandom", cfg->mCbxRandom->isChecked());
+  config.writeEntry("ZoomImages", cfg->mCbxZoom->isChecked());
+  config.writeEntry("PrintName",  cfg->mCbxShowName->isChecked());
+  config.writeEntry("Delay", cfg->mDelay->value());
+  config.writePathEntry("Directory", cfg->mDirChooser->url().path());
+  config.writeEntry("SubDirectory", cfg->mCbxSubdirectory->isChecked());
+  config.writeEntry("RandomPosition", cfg->mCbxRandomPosition->isChecked());
 
-  config->sync();
+  config.sync();
 
   if (mSaver)
   {
@@ -977,6 +1005,12 @@ void kSlideShowSetup::writeSettings()
 
 
 //-----------------------------------------------------------------------------
+
+void kSlideShowSetup::slotDirSelected(const KUrl &)
+{
+      writeSettings();
+}
+
 void kSlideShowSetup::slotDirSelected(const QString &)
 {
   writeSettings();
@@ -994,13 +1028,6 @@ void kSlideShowSetup::slotOk()
 //-----------------------------------------------------------------------------
 void kSlideShowSetup::slotHelp()
 {
-  KAboutData aboutData( "kslideshow.kss", I18N_NOOP("SlideShow"),
-                       version, description, KAboutData::License_GPL,
-                       "(c) 1999-2003, The KDE Team" );
-  aboutData.addAuthor("Stefan Taferner", 0, "taferner@kde.org");
-  aboutData.addAuthor("Chris Howells", 0, "howells@kde.org");
-  aboutData.addAuthor("Sven Leiber", 0, "s.leiber@web.de");
-
-  KAboutApplication mAbout(&aboutData, this, 0);
+  KAboutApplicationDialog mAbout(s_aboutData, this);
   mAbout.exec();
 }
